@@ -1,3 +1,4 @@
+#' @rdname find_data
 #' @title Extract data from a model object
 #' @description Attempt to reconstruct the data used to create a model object
 #' @param model The model object.
@@ -36,7 +37,7 @@ find_data.default <- function(model, env = parent.frame(), ...) {
         if (!is.null(model[["call"]][["subset"]])) {
             subs <- try(eval(model[["call"]][["subset"]], dat), silent = TRUE)
             if (inherits(subs, "try-error")) {
-                subs <- try(get("subs", env), silent = TRUE)
+                subs <- try(eval(model[["call"]][["subset"]], env), silent = TRUE)
                 if (inherits(subs, "try-error")) {
                     subs <- TRUE
                     warning("'find_data()' cannot locate variable(s) used in 'subset'")
@@ -63,7 +64,19 @@ find_data.data.frame <- function(model, ...) {
 
 #' @rdname find_data
 #' @export
-find_data.lm <- find_data.default
+find_data.crch <- find_data.default
+
+#' @rdname find_data
+#' @export
+find_data.glimML <- function(model, ...) {
+    requireNamespace("methods", quietly = TRUE)
+    methods::slot(model, "data")
+}
+
+.find_data_glimQL <- function(model, env = parent.frame(), ...) {
+    requireNamespace("methods", quietly = TRUE)
+    methods::slot(model, "fm")$data
+}
 
 #' @rdname find_data
 #' @export
@@ -71,9 +84,16 @@ find_data.glm <- find_data.default
 
 #' @rdname find_data
 #' @export
-find_data.svyglm <- function(model, ...) {
-    data <- model[["data"]]
-    data
+find_data.hxlr <- find_data.default
+
+#' @rdname find_data
+#' @export
+find_data.lm <- find_data.default
+
+#' @rdname find_data
+#' @export
+find_data.mca <- function(model, env = parent.frame(), ...) {
+    eval(model[["call"]][["df"]], envir = env)
 }
 
 #' @rdname find_data
@@ -85,11 +105,33 @@ find_data.merMod <- function(model, env = parent.frame(), ...) {
 
 #' @rdname find_data
 #' @export
-find_data.crch <- find_data.default
+find_data.svyglm <- function(model, env = parent.frame(), ...) {
+    dat <- model[["data"]]
+    # handle subset
+    if (!is.null(model[["call"]][["subset"]])) {
+        subs <- try(eval(model[["call"]][["subset"]], dat), silent = TRUE)
+        if (inherits(subs, "try-error")) {
+            subs <- try(eval(model[["call"]][["subset"]], env), silent = TRUE)
+            if (inherits(subs, "try-error")) {
+                subs <- TRUE
+                warning("'find_data()' cannot locate variable(s) used in 'subset'")
+            }
+        }
+        dat <- dat[subs, , drop = FALSE]
+    }
+    # handle na.action
+    if (!is.null(model[["na.action"]])) {
+        dat <- dat[-model[["na.action"]], , drop = FALSE]
+    }
+
+    return(dat)
+}
 
 #' @rdname find_data
 #' @export
-find_data.hxlr <- find_data.default
+find_data.train <- function(model, ...) {
+    model[["trainingData"]]
+}
 
 #' @rdname find_data
 #' @export
@@ -104,9 +146,3 @@ find_data.vgam <- function(model, env = parent.frame(), ...) {
 #' @rdname find_data
 #' @export
 find_data.vglm <- find_data.vgam
-
-#' @rdname find_data
-#' @export
-find_data.mca <- function(model, env = parent.frame(), ...) {
-    eval(model[["call"]][["df"]], envir = env)
-}
