@@ -1,6 +1,6 @@
 # @rdname prediction
 # @export
-prediction.bigglm <- 
+.prediction_bigglm <- 
 function(model, 
          data = find_data(model, parent.frame()), 
          at = NULL, 
@@ -33,11 +33,18 @@ function(model,
         }
     }
     
-    # obs-x-(ncol(data)+2) data frame
+    # variance(s) of average predictions
+    vc <- NA_real_
+    
+    # output
     structure(pred, 
               class = c("prediction", "data.frame"),
-              row.names = seq_len(nrow(pred)),
               at = if (is.null(at)) at else at_specification,
-              model.class = class(model),
-              type = type)
+              type = type,
+              call = if ("call" %in% names(model)) model[["call"]] else NULL,
+              model_class = class(model),
+              row.names = seq_len(nrow(pred)),
+              vcov = vc,
+              jaccobian = NULL,
+              weighted = FALSE)
 }
