@@ -1,6 +1,6 @@
 # @rdname prediction
 # @export
-prediction.vglm <- 
+.prediction_vglm <- 
 function(model, 
          data = find_data(model, parent.frame()), 
          at = NULL, 
@@ -54,12 +54,19 @@ function(model,
         pred[["fitted"]] <- pred[[ w[1L] ]]
     }
     
-    # obs-x-(ncol(data)+2) data frame
+    # variance(s) of average predictions
+    vc <- NA_real_
+    
+    # output
     structure(pred, 
               class = c("prediction", "data.frame"),
-              row.names = seq_len(nrow(pred)),
               at = if (is.null(at)) at else at_specification,
-              model.class = class(model),
               type = type,
-              category = category)
+              call = if ("call" %in% names(model)) model[["call"]] else NULL,
+              model_class = class(model),
+              row.names = seq_len(nrow(pred)),
+              vcov = vc,
+              jacobian = NULL,
+              category = category,
+              weighted = FALSE)
 }
